##Diagrama ombrotémico de Walter y Lieth
##Francisco Alcaraz Ariza (Murcia, Spain)

## Versión 2.1 (27 de enero de 2010)
# Cambios en la versión respecto a 2.0
#Reducidos los tamaños de letras para uso en ultraportátiles

## Versión 2.0 (20 de agosto de 2009)
#Cambios en la versión respecto a 1.5:
# Arreglado para precipitaciones mayores de 300 m de modo que el gráfico no interfiera con los rótulos, ahora se pueden hacer
#   los diagramas de las estaciones más lluviosas del mundo
#Las etiquetas de los dos ejes verticales se ajustan al límite superior de los mismos y se separan de aquellos para evitar interferir 
#  con las marcas laterales

## Versión 1.5
# Cambios en la versión respecto a 1.3 
# Añadido en "par" la opción family="HersheySans" que parece resolver los problemas de caracteres acentuados (ahora introducidos con secuencias \\) y la salida no de puntos en los pdf para las épocas secas
# Recortado texto en leyenda para heladas
# Cabecera estación centrada, segunda línea separada un poco
# Optimizado salidas a archivos pdf y svg con CairoDevice

## Versión 1.3
# Cambios en la versión respecto a 1.2:
# Añadida leyenda heladas, resuelto layout para evitar que funciones llamadas a continuación creen gráficos en zonas 2 ó 3
# Cambiado color para representar las heladas seguras, se confundía demasiado con el de las probables.
# Arreglado problema en etiqueta T ºC para salida svg

# Versión 1.2
#Cambios respecto a 1.0:
#Corregido bug de redondeo del texto en T y P media anual

diagram<-function(file,lang="es",margen=c(2,1,5,2),mlab="",pcol="#005ac8", tcol="#e81800", pfcol="cyan",sfcol="red",shem=FALSE) {
par(mar=margen,pty="s",las=1,new=FALSE,cex.main=1,family="HersheySans")


#Cargamos los ficheros
paste(file,".dat",sep="")->file1
paste(file,".loc",sep="")->file2
read.table(file1,header=T)->dat
read.table(file2,header=T)->loc

#Controlamos número de columnas en el fichero de datos; si son sólo dos la segunda deben ser las temperaturas medias, por lo que el diagrama se realizará directamente, pero no se podrán poner las heladas
nc<-ncol(dat)

#Etiquetas de los meses, cabecera, etc.; diferenciadas, en su caso, según el idioma
if(lang=="en") mlab=c("J","F","M","A","M","J","J","A","S","O","N","D")
else mlab=c("E","F","M","A","M","J","J","A","S","O","N","D")
if(lang=="en") titulo="Ombrothermic diagram" else titulo="Diagrama ombrot\\'ermico"
if(lang=="en") seco<-"Dry" else seco<-"Seco"
if(lang=="en") humedo<-"Humid" else humedo<-"H\\'umedo"
hiperhumedo<-"P>100 mm"
if(lang=="en") segura<-"Sure frozen" else segura<-"H. segura"
if(lang=="en") probable<-"Possible frozen" else probable<-"H. probable"
hem=loc[1,7]
if(hem=="2") shem=TRUE 
else shem=FALSE
if(shem) {#Hemisferio sur: desplazar los puntos y etiquetas medio año
m1 <- dat[1:6,]
m2 <- dat[7:12,]
dat<-rbind(m2,m1)
mlab<-c(mlab[7:12],mlab[1:6])
}

#Enlazamos los nombres de las variables a los de las columnas de los ficheros
attach(dat)
attach(loc)

#Temperaturas medias mensuales
if(nc==4) {tm<-(tma + tmi)/2}
#En caso contrario (sólo dos columnas en el fichero) es evidente que la segunda columna, interpretada por «attach» es tm

pmax <- max(pm) #precipitación máxima
  ymax <- 60  #máxima ordenada por defecto
cat("ymax=",ymax,"\n");
  if((pmax > 250) && (pmax<=300)) ymax <- 60 + 10*floor((pmax+100)/200)
  if((pmax > 300) && pmax<=800) ymax <- 50 + 10*floor((pmax+100)/200)
  if(pmax > 800) ymax <- 60 + 10*floor((pmax+100)/200) 

cat("pmax=",pmax,"ymax=",ymax,"\n");
ymin <- min(-1.5,min(tm)) #mínima ordenada sin redondear
  #ejes:
  if(ymin < -1.5) {
    ymin<-floor(ymin/10)*10 #mínima ordenada redondeada
cat("ymin=",ymin,"\n")
    labT <- paste(ymin)
    labP <- ""
    if(ymin < -10) {
      for(i in (ymin/10+1):-1) {
        labT <- c(labT,i*10)
        labP <- c(labP,"")
cat(i,i*10,"\n",labT,"\n",labP,"\n")
      }
    }
    labT <- c(labT,"0","10","20","30","40","50","")
    labP <- c(labP,"0","20","40","60","80","100","300")
  }
  else {
    labT <- c("0","10","20","30","40","50","")
    labP <- c("0","20","40","60","80","100","300")
  }
  if(ymax > 60) {
    for(i in 6:(ymax/10-1)) {
      labT <- c(labT,"")
      labP <- c(labP,100*(2*i-7))
    }
  }

#Iniciamos la salida gráfica, primero dividimos la pantalla en tres zonas, una para el gráfico y dos para la leyenda
m<-matrix(c(1,1,1,1,1,1,2,3),4,2,byrow=T)
layout(m)
#La parte más importante de la salida se dedica al diagrama
  plot(0:13-0.5,c(tm[12],tm[1:12],tm[1]),xlim=c(0,12),ylim=c(ymin,ymax),type="l",col=tcol,xaxs="i",yaxs="i",xaxp=c(0,12,12),xlab="",
ylab="",xaxt="n",yaxt="n",bty="n",main=titulo,lty="dotted",cex=0.8)
  lmin <- ymin #mínima ordenada a rotular
  if(lmin==-1.5) lmin=0
  axis(2,((lmin/10):(ymax/10))*10,labels=labT,col.axis=tcol,cex.axis=0.8)
  axis(4,((lmin/10):(ymax/10))*10,labels=labP,col.axis=pcol,cex.axis=0.8)
  mtext("T \\deC",2,col=tcol,las=1,line=3,adj=1,at=ymax,cex=0.6)
  mtext("P mm",4,col=pcol,las=1,line=3,adj=-0.1,at=ymax,cex=0.6)
  if(ymin< -1.5){
	abline(ymin+1.5,0);
	abline(0,0,lty="dotted")}
	else abline(0,0)
 
#rótulos:
if(hem==1) z<-"N" else z<-"S"
if(long3==1) zz<-"E" else zz<-"W"

mtext(paste(est," [",alt," m], ",round(sum(tm)/12,digit=1)," \\deC ",round(sum(pm),digit=1),"  mm",sep=""),line=0.5,adj=0.5,cex=0.6)

#Texto en la cabecera del diagrama
if(yeart==yearp) mtext(paste("   [",yeart,"] ", lat1,"\\de ",lat2,"' ",z,", ",long1,"\\de ",long2,"' ",zz,sep=""),line=-0.9,adj=0,cex=0.6)
else mtext(paste("   [",yeart,"-",yearp,"] ", lat1,"\\de ",lat2,"' ",z,", ",long1,"\\de ",long2,"' ",zz,sep=""),line=-0.9,adj=0,cex=0.6)

#Líneas, polígonos rellenos, etc.
  x <- 0:13-0.5
  pm2 <- c(pm[12],pm[1:12],pm[1])
  if(pmax<=100) {
    xl <- x
    yl <- c(pm[12],pm[1:12],pm[1])/2
    n2 <- 14
  }
  else { #cambio de escala en prec. > 100
    xp <- numeric(30)
    yp <- numeric(30)
    xl <- numeric(25)
    yl <- numeric(25)
    n <- 0
    n2 <- 0
    gr <- FALSE
    if(pm2[1]>100) { #primer punto
      n <- n+1
      xp[n] <- x[1]
      yp[n] <- 50
      n <- n+1
      xp[n] <- x[1]
      yp[n] <- 50 + (pm2[1]-100)/20
      n2 <- n2+1
      xl[n2] <- x[1]
      yl[n2] <- 50
      gr <- TRUE
    }
    else {
      n2 <- n2+1
      xl[n2] <- x[1]
      yl[n2] <- pm2[1]/2
    }
    for(i in 2:14) {  #demás puntos
      if(gr) {  #si p anterior > 100
        n <- n+1
        if(pm2[i]>100) {
          xp[n] <- x[i]
          yp[n] <- 50 + (pm2[i]-100)/20
        }
        else {
          xp[n] <- x[i-1] + (100-pm2[i-1])/(pm2[i]-pm2[i-1])
          yp[n] <- 50
          n2 <- n2+1
          xl[n2] <- xp[n]
          yl[n2] <- 50
          n <- n+1
          xp[n] <- NA
          yp[n] <- NA
          n2 <- n2+1
          xl[n2] <- x[i]
          yl[n2] <- pm2[i]/2
          gr <- FALSE
        }
      }
      else {  # p anterior <=100
        if(pm2[i]>100) { #si p > 100
          n <- n+1
          xp[n] <- x[i-1] + (100-pm2[i-1])/(pm2[i]-pm2[i-1])
          yp[n] <- 50
          if(xl[n2]!=x[i-1]) {  #no repetir puntos!
            n2 <- n2+1
            xl[n2] <- x[i-1]
            yl[n2] <- pm2[i-1]/2
          }
          n2 <- n2+1
          xl[n2] <- xp[n]
          yl[n2] <- 50
          n <- n+1
          xp[n] <- x[i]
          yp[n] <- 50 + (pm2[i]-100)/20
          gr <- TRUE
        }
        else { # p <=100
          n2 <- n2+1
          xl[n2] <- x[i]
          yl[n2] <- pm2[i]/2
        }
      }
    }
    if(!is.na(yp[n])) {  #cerrar último polígono
      n <- n+1
      xp[n] <- xp[n-1]
      yp[n] <- 50
      n2 <- n2+1
      xl[n2] <- 12.5
      yl[n2] <- 50
    }
    polygon(xp[1:n],yp[1:n],col=pcol,border=pcol)
  }
  #tramas:
  pi <- approx(xl[1:n2],yl[1:n2],n=66)$y
  ti <- approx(x,c(tm[12],tm[1:12],tm[1]),n=66)$y
  d <- pi - ti
  xi <- (1:66)/5-0.7
  xw <- subset(xi,d>0) #periodo húmedo
  y1 <- subset(pi,d>0)
  y2 <- subset(ti,d>0)
  if(length(xw)>0) segments(xw,y1,xw,y2,col=pcol,lty=1,lwd=1)
  xw <- subset(xi,d<0) #periodo seco
  y1 <- subset(pi,d<0)
  y2 <- subset(ti,d<0)
  if(length(xw)>0) segments(xw,y1,xw,y2,col=tcol,lty=3,lwd=2)
  #curvas de P y T:
  lines(xl[1:n2],yl[1:n2],col=pcol,lwd=2)
  lines(x,c(tm[12],tm[1:12],tm[1]),col=tcol,lwd=2,lty="dotted")

  #marcar límites de los meses:
  for(i in 0:13)
	if(lmin> -1.5)
	segments(i,0,i,-1.5) else
	segments(i,lmin+1.5,i,lmin)

#Se aplica la rutina de heladas siempre que tengamos las temperaturas mínimas (fichero con 4 columnas)
#heladas seguras
if(nc==4)
{for(i in 1:12) 
	if(tmi[i]<=0)
		if(lmin> -1.5) rect(i-1,-1.5,i,0,col=sfcol) else rect(i-1,lmin,i,lmin+1.5,col=sfcol)
  for(i in 1:12) 
	if(tmi[i]>0 && tmia[i]<=0)
		if(lmin> -1.5) rect(i-1,-1.5,i,0,col=pfcol) else rect(i-1,lmin,i,lmin+1.5,col=pfcol)}
else{}
	#rect(i-1,-1.5,i,0,col=pfcol)
  #rótulos meses:
  mtext(mlab,1,las=1,line=0.5,adj=0.5,at=x[2:13],cex=0.6)



#Segunda parte de la salida gráfica para la primera parte de la leyenda
margen=c(2,7,1,2)
par(mar=margen,pty="s",las=1,new=FALSE)
plot.new()
rect(0.01,0.285,0.29,0.475,col=tcol,density=20,angle=90,lw=2,lty=3)
rect(0,0.28,0.3,0.48,col=0)
text(0.36,0.40,seco,adj=0,font=2,cex=0.8)
rect(0,0.55,0.3,0.75,col=pcol,density=20,angle=90)
rect(0,0.55,0.3,0.75,col=0)
text(0.36,0.65,humedo,adj=0,font=2,cex=0.8)
rect(0,0.80,0.3,1,col=pcol)
text(0.36,0.90,hiperhumedo,adj=0,font=2,cex=0.8)
#Heladas seguras si hay datos de mínimas
if(nc==4){
hs=0
for(i in 1:12)
  if(tmi[i]<=0) hs=hs+1
else{}
if (hs>0){
rect(0,0,0.3,0.2,col=sfcol)
text(0.36,0.09,segura,adj=0,font=2)}
else{}}

#Tercera parte de la salida gráfica para la segunda parte de la leyenda
margen=c(0.1,0,0.1,0)
par(mar=margen,pty="s",las=1,new=FALSE)
plot.new()
segments(0.1,0.6,0.4,0.6,col="red",lty="dotted",lwd=2)
text(0.5,0.6,"T",font=2,cex=0.8,adj=0)
segments(0.1,0.8,0.4,0.8,col="blue",lwd=2)
text(0.5,0.8,"P",font=2,cex=0.8,adj=0)
#Heladas probables si es que hay datos de mínimas y alguna helada
if(nc==4){
hp=0
for(i in 1:12)
  if(tmi[i]>0 && tmia[i]<=0) hp=hp+1
else{}
if (hp>0){
rect(0.1,0.15,0.35,0.3,col=pfcol)
text(0.42,0.22,probable,adj=0,font=2,cex=0.8)}
else{}}

#Restablecemos parámetros gráficos anteriores, para evitar efectos indeseados cuando otras funciones tengan una salida gráfica
#invisible()
#m<-matrix(c(1,1,1,1,1,1,1,1),4,2,byrow=T)
#layout(m)
}