##Diagrama de Balance hídrico
##Francisco Alcaraz Ariza (Murcia, Spain)

##Versión 1.5 (20 de agosto de 2009)
#Cambios respecto a 1.1
#Corregidos diagramas donde los polígonos se superponen a los rótulos (zonas con mucha lluvia), incluidas las
#estaciones más lluviosas del mundo

## Versión 1.1
# Cambios en la versión respecto a 1.0
# Resuelto layout para evitar que funciones llamadas a continuación creen gráficos en zonas 2 ó 3
diagrambh<-function(file,lang="es",res=200,dat,loc,margen=c(2,1,5,2),mlab="",pcol="#005ac8", tcol="#e81800", pfcol="cyan",sfcol="#0eb6d7",etpcol="black",etpscol="grey",shem=FALSE) {
#plot.new()
par(mar=margen,pty="s",las=1,new=FALSE,family="HersheySans")
#################################################
#Limpiamos dat y loc de memoria por si se hubieran cargado con anterioridad
#if(exists("dat")) rm(dat)
#if(exists("loc")) rm(loc)

#Creamos las variables que vamos a utilizar
f<-1:12; e<-1:12; s<-1:12; Lr<-0;etp<-1:12;etr<-1:12;vr<-1:12;r<-1:12;sp<-1:12;d<-1:12;Etp<-0;latitud<-0

#Les damos valor cero a las variables
for(i in 1:12){
f[i]<-0; e[i]<-0;s[i]<-0;etp[i]<-0;etr[i]<-0;vr[i]<-0;r[i]<-0;sp[i]<-0;d[i]<-0}

#Cargamos los ficheros
paste(file,".dat",sep="")->file1
paste(file,".loc",sep="")->file2
read.table(file1,header=T)->dat
read.table(file2,header=T)->loc

#Controlamos número de columnas en el fichero de datos
#Si son sólo dos la segunda deben ser las temperaturas medias
nc<-ncol(dat)

#etiquetas de los meses (incluyendo las de inglés)
mlab<-1:12 
mlab[2]<-" F";mlab[3]<-" M";mlab[4]<-" A";mlab[5]<-" M";mlab[6]<-" J";mlab[7]<-" J";mlab[8]<-" A";mlab[9]<-" S";mlab[10]<-" O";mlab[11]<-" N";mlab[12]<-" D"
if(lang=="en") mlab[1]<-" J" else mlab[1]<-" E"

#Título del diagrama
if(lang=="en") titulo="Hydric balance diagram" else titulo="Diagrama de balance h\\'idrico"

#Caracterizamos el hemisferio a partir de los ficheros de datos
hem=loc[1,7]
if(hem=="2") shem=TRUE 
else shem=FALSE
if(shem) {#Hemisferio sur: desplazar los datos medio año
m1 <- dat[1:6,]
m2 <- dat[7:12,]
dat<-rbind(m2,m1)
mlab<-c(mlab[7:12],mlab[1:6])
}
#Captamos desde los ficheros los nombres de las variables
attach(dat)
attach(loc)

#Temperaturas medias mensuales, primero si dat tenía sólo dos columnas
if(nc==4) {tm<-(tma+tmi)/2}
#En caso contrario es evidente que la segunda columna, interpretada por «attach» es tm
Tm=round(sum(tm)/12,digit=1)
Pm=round(sum(pm),digit=1)


##############################################################
############### Evapotranspiración potencial #################
##############################################################
#La latitud tiene dos componentes, lat1(grados), lat2(minutos), vamos a redondearla para hacer el cálculo, pues Thornwtwaite sólo usa los grados
if(lat2<=30) lat1->latitud else lat1+1->latitud 

#El componente «f» de la etp, sin fórmula, valores directos dependientes del hemisferio
if(hem=="2") shem=TRUE else shem=FALSE
if(shem) { #shem=TRUE, es decir, hemisferio es el sur
 if(latitud<5)
  {f[1]<-32.24;f[2]<-26.32;f[3]<-32.24;f[4]<-30.3;f[5]<-32.24;
  f[6]<-30.3;f[7]<-32.24;f[8]<-32.24;f[9]<-30.3;f[10]<-32.24;
  f[11]<-30.3;f[12]<-32.24} else
 if(5 <=latitud && latitud<10)
  {f[1]<-31*1.06;f[2]<-28*0.91;f[3]<-31*1.04;f[4]<-30*1.00;f[5]<-31*1.02;
  f[6]<-30*0.99;f[7]<-31*1.02;f[8]<-31*1.03;f[9]<-30*1.00;f[10]<-31*1.05;
  f[11]<-30*1.03;f[12]<-31*1.06} else
 if(10<=latitud && latitud<15)
  {f[1]<-31*1.08;f[2]<-28*0.97;f[3]<-31*1.05;f[4]<-30*0.99;f[5]<-31*1.01;
  f[6]<-30*0.96;f[7]<-31*1.00;f[8]<-31*1.01;f[9]<-30*1.00;f[10]<-31*1.06;
  f[11]<-30*1.05;f[12]<-31*1.10} else
 if(15<=latitud && latitud<20)
  {f[1]<-31*1.12;f[2]<-28*0.98;f[3]<-31*1.05;f[4]<-30*0.98;f[5]<-31*0.98;
  f[6]<-30*0.94;f[7]<-31*0.97;f[8]<-31*1.00;f[9]<-30*1.00;f[10]<-31*1.07;
  f[11]<-30*1.07;f[12]<-31*1.07} else
 if(20<=latitud && latitud<25)
  {f[1]<-31*1.14;f[2]<-28*1.00;f[3]<-31*1.05;f[4]<-30*0.97;f[5]<-31*0.96;
  f[6]<-30*0.91;f[7]<-31*0.95;f[8]<-31*0.99;f[9]<-30*1.00;f[10]<-31*1.08;
  f[11]<-30*1.09;f[12]<-31*1.15} else
 if(25<=latitud && latitud<30)
  {f[1]<-31*1.17;f[2]<-28*1.01;f[3]<-31*1.05;f[4]<-30*0.96;f[5]<-31*0.94;
  f[6]<-30*0.88;f[7]<-31*0.93;f[8]<-31*0.98;f[9]<-30*1.00;f[10]<-31*1.10;
  f[11]<-30*1.11;f[12]<-31*1.18} else
 if(30<=latitud && latitud<35)
  {f[1]<-31*1.20;f[2]<-28*1.03;f[3]<-31*1.06;f[4]<-30*0.95;f[5]<-31*0.92;
  f[6]<-30*0.85;f[7]<-31*0.90;f[8]<-31*0.96;f[9]<-30*1.00;f[10]<-31*1.12;
  f[11]<-30*1.14;f[12]<-31*1.21} else
 if(35<=latitud && latitud<40)
  {f[1]<-31*1.23;f[2]<-28*1.04;f[3]<-31*1.06;f[4]<-30*0.94;f[5]<-31*0.89;
  f[6]<-30*0.82;f[7]<-31*0.87;f[8]<-31*0.94;f[9]<-30*1.00;f[10]<-31*1.13; 
  f[11]<-30*1.17;f[12]<-31*1.25} else
 if(40<=latitud && latitud<42)
  {f[1]<-31*1.27;f[2]<-28*1.06;f[3]<-31*1.07;f[4]<-30*0.93;f[5]<-31*0.86;
  f[6]<-30*0.78;f[7]<-31*0.84;f[8]<-31*0.92;f[9]<-30*1.00;f[10]<-31*1.15;
  f[11]<-30*1.20;f[12]<-31*1.29} else
 if(42<=latitud && latitud<44)
  {f[1]<-31*1.28;f[2]<-28*1.07;f[3]<-31*1.07;f[4]<-30*0.92;f[5]<-31*0.85;
  f[6]<-30*0.76;f[7]<-31*0.82;f[8]<-31*0.92;f[9]<-30*1.00;f[10]<-31*1.16;
  f[11]<-30*1.22;f[12]<-31*1.31} else
 if(44<=latitud && latitud<46)
  {f[1]<-31*1.30;f[2]<-28*1.08;f[3]<-31*1.07;f[4]<-30*0.92;f[5]<-31*0.83;
  f[6]<-30*0.74;f[7]<-31*0.81;f[8]<-31*0.91;f[9]<-30*0.99;f[10]<-31*1.17;
  f[11]<-30*1.23;f[12]<-31*1.33} else
 if(46<=latitud && latitud<48)
  {f[1]<-31*1.32;f[2]<-28*1.10;f[3]<-31*1.07;f[4]<-30*0.91;f[5]<-31*0.82;
  f[6]<-30*0.72;f[7]<-31*0.79;f[8]<-31*0.90;f[9]<-30*0.99;f[10]<-31*1.17;
  f[11]<-30*1.25;f[12]<-31*1.35} else
 if(48<=latitud && latitud<50)
  {f[1]<-31*1.34;f[2]<-28*1.11;f[3]<-31*1.08;f[4]<-30*0.90;f[5]<-31*0.80;
  f[6]<-30*0.70;f[7]<-31*0.76;f[8]<-31*0.89;f[9]<-30*0.99;f[10]<-31*1.18;
  f[11]<-30*1.27;f[12]<-31*1.37} else
 if(latitud>=50)
  {f[1]<-42.47;f[2]<-30.8;f[3]<-33.48;f[4]<-26.7;f[5]<-23.87;
  f[6]<-20.1;f[7]<-22.94;f[8]<-27.28;f[9]<-29.7;f[10]<-36.98;
  f[11]<-38.7;f[12]<-43.71}}

else #shem=FALSE, estamos en el hemisferio norte
 if(latitud<5) 
   {f[1]<-32.24;f[2]<-26.32;f[3]<-32.24;f[4]<-30.3;f[5]<-32.24;f[6]<-30.3;
   f[7]<-32.24;f[8]<-32.24;f[9]<-30.3;f[10]<-32.24;f[11]<-30.3;f[12]<-32.24} else
 if(5<=latitud && latitud<10)
   {f[1]<-31.62;f[2]<-26.04;f[3]<-31.93;f[4]<-30.6;f[5]<-32.86;f[6]<-30.9;
   f[7]<-32.86;f[8]<-32.55;f[9]<-30.3;f[10]<-31.93;f[11]<-29.7;f[12]<-31.62} else
 if(10<=latitud && latitud<15)
   {f[1]<-31.00;f[2]<-25.48;f[3]<-31.93;f[4]<-30.90;f[5]<-33.48;f[6]<-31.80;
   f[7]<-33.48;f[8]<-33.17;f[9]<-30.6;f[10]<-31.62;f[11]<-29.4;f[12]<-30.69} else
 if(15<=latitud && latitud<20)
   {f[1]<-30.07;f[2]<-25.48;f[3]<-31.93;f[4]<-31.2;f[5]<-34.41;f[6]<-32.4;
   f[7]<-32.72;f[8]<-33.48;f[9]<-30.6;f[10]<-31.31;f[11]<-28.5;f[12]<-30.07} else
 if(20<=latitud && latitud<25)
   {f[1]<-29.45;f[2]<-25.2;f[3]<-31.93;f[4]<-31.5;f[5]<-35.03;f[6]<-33.30;
   f[7]<-35.34;f[8]<-34.41;f[9]<-30.6;f[10]<-31;f[11]<-27.9;f[12]<-29.14} else
 if(latitud==25)
   {f[1]<-27.9;f[2]<-26.7;f[3]<-30.9;f[4]<-31.8;f[5]<-34.5;f[6]<-34.2;
   f[7]<-35.1;f[8]<-33.6;f[9]<-30.6;f[10]<-29.7;f[11]<-27.3;f[12]<-27.3} else
 if(latitud==26)
   {f[1]<-27.6;f[2]<-26.4;f[3]<-30.9;f[4]<-32.1;f[5]<-34.8;f[6]<-34.5;
   f[7]<-35.1;f[8]<-33.6;f[9]<-30.6;f[10]<-29.7;f[11]<-27.3;f[12]<-27.3} else
 if(latitud==27)
   {f[1]<-27.6;f[2]<-26.4;f[3]<-30.9;f[4]<-32.1;f[5]<-34.8;f[6]<-34.5;
   f[7]<-35.4;f[8]<-33.9;f[9]<-30.6;f[10]<-29.7;f[11]<-27.0;f[12]<-27.0} else
 if(latitud==28)
   {f[1]<-27.3;f[2]<-26.4;f[3]<-30.9;f[4]<-32.1;f[5]<-35.1;f[6]<-34.8;
   f[7]<-35.4;f[8]<-33.9;f[9]<-30.9;f[10]<-29.4;f[11]<-27.0;f[12]<-27.0} else
 if(latitud==29)
   {f[1]<-27.3;f[2]<-26.1;f[3]<-30.9;f[4]<-32.1;f[5]<-35.1;f[6]<-34.8;
   f[7]<-35.7;f[8]<-33.9;f[9]<-30.9;f[10]<-29.4;f[11]<-26.7;f[12]<-26.7} else
 if(latitud==30)
   {f[1]<-27.0;f[2]<-26.1;f[3]<-30.9;f[4]<-32.4;f[5]<-35.4;f[6]<-35.1;
   f[7]<-36.0;f[8]<-34.2;f[9]<-30.9;f[10]<-29.4;f[11]<-26.7;f[12]<-26.4} else
 if(latitud==31)
   {f[1]<-27;f[2]<-26.1;f[3]<-30.9;f[4]<-32.4;f[5]<-35.4;f[6]<-35.1;
   f[7]<-36;f[8]<-34.2;f[9]<-30.9;f[10]<-29.4;f[11]<-26.4;f[12]<-26.4} else
 if(latitud==32)
   {f[1]<-26.7;f[2]<-25.8;f[3]<-30.9;f[4]<-32.4;f[5]<-35.7;f[6]<-35.4;
   f[7]<-36.3;f[8]<-34.5;f[9]<-30.9;f[10]<-29.4;f[11]<-26.4;f[12]<-26.1} else
 if(latitud==33)
   {f[1]<-26.4;f[2]<-25.8;f[3]<-30.9;f[4]<-32.7;f[5]<-35.7;f[6]<-35.7;
   f[7]<-36.3;f[8]<-34.5;f[9]<-30.9;f[10]<-29.1;f[11]<-26.1;f[12]<-25.8} else
 if(latitud==34)
   {f[1]<-26.4;f[2]<-25.8;f[3]<-30.9;f[4]<-32.7;f[5]<-36;f[6]<-36;
   f[7]<-36.6;f[8]<-34.8;f[9]<-30.9;f[10]<-29.1;f[11]<-26.1;f[12]<-25.8} else
 if(latitud==35)
   {f[1]<-26.1;f[2]<-25.5;f[3]<-30.9;f[4]<-32.7;f[5]<-36.3;f[6]<-36.3;
   f[7]<-36.9;f[8]<-34.8;f[9]<-30.9;f[10]<-29.1;f[11]<-25.8;f[12]<-25.5} else
 if(latitud==36)
   {f[1]<-26.1;f[2]<-25.5;f[3]<-30.9;f[4]<-33;f[5]<-36.3;f[6]<-36.6;
   f[7]<-37.2;f[8]<-34.8;f[9]<-30.9;f[10]<-29.1;f[11]<-25.8;f[12]<-25.2} else
 if(latitud==37)
   {f[1]<-25.8;f[2]<-25.5;f[3]<-30.9;f[4]<-33.0;f[5]<-36.6;f[6]<-36.9;
   f[7]<-37.5;f[8]<-35.1;f[9]<-30.9;f[10]<-29.1;f[11]<-25.5;f[12]<-24.9} else
 if(latitud==38)
   {f[1]<-25.5;f[2]<-25.2;f[3]<-30.9;f[4]<-33;f[5]<-36.9;f[6]<-37.2;
   f[7]<-37.5;f[8]<-35.1;f[9]<-31.2;f[10]<-28.8;f[11]<-25.2;f[12]<-24.9} else
 if(latitud==39)
   {f[1]<-25.5;f[2]<-25.2;f[3]<-30.9;f[4]<-33.3;f[5]<-36.9;f[6]<-37.2;
   f[7]<-37.8;f[8]<-35.4;f[9]<-31.2;f[10]<-28.8;f[11]<-25.2;f[12]<-24.6} else
 if(latitud==40)
   {f[1]<-25.2;f[2]<-24.9;f[3]<-30.9;f[4]<-33.3;f[5]<-37.2;f[6]<-37.5;
   f[7]<-38.1;f[8]<-35.4;f[9]<-31.2;f[10]<-28.8;f[11]<-24.9;f[12]<-24.3} else
 if(latitud==41)
   {f[1]<-24.9;f[2]<-24.9;f[3]<-30.9;f[4]<-33.3;f[5]<-37.5;f[6]<-37.8;
   f[7]<-38.1;f[8]<-35.7;f[9]<-31.2;f[10]<-28.8;f[11]<-24.6;f[12]<-24.0} else
 if(latitud==42)
   {f[1]<-24.6;f[2]<-24.6;f[3]<-30.9;f[4]<-33.6;f[5]<-37.8;f[6]<-38.1;
   f[7]<-38.4;f[8]<-35.7;f[9]<-31.2;f[10]<-28.5;f[11]<-24.6;f[12]<-23.7} else
 if(latitud==43)
   {f[1]<-24.3;f[2]<-24.6;f[3]<-30.6;f[4]<-33.6;f[5]<-37.8;f[6]<-38.4;
   f[7]<-38.7;f[8]<-36;f[9]<-31.2;f[10]<-28.5;f[11]<-24.3;f[12]<-23.1} else
 if(latitud==44)
   {f[1]<-0.81*31;f[2]<-0.82*28;f[3]<-1.02*31;f[4]<-1.13*30;f[5]<-1.27*31;
   f[6]<-1.29*30;f[7]<-1.30*31;f[8]<-1.20*31;f[9]<-1.04*30;f[10]<-0.95*31;
   f[11]<-0.8*30;f[12]<-0.76*31} else
 if(latitud==45)
   {f[1]<-0.8*31;f[2]<-0.81*28;f[3]<-1.02*31;f[4]<-1.13*30;f[5]<-1.28*31;
   f[6]<-1.29*30;f[7]<-1.31*31;f[8]<-1.21*31;f[9]<-1.04*30;f[10]<-0.94*31;
   f[11]<-0.79*30;f[12]<-0.75*31} else
 if(latitud==46)
   {f[1]<-0.79*31;f[2]<-0.81*28;f[3]<-1.02*31;f[4]<-1.13*30;f[5]<-1.29*31;
   f[6]<-1.31*30;f[7]<-1.32*31;f[8]<-1.22*31;f[9]<-1.04*30;f[10]<-0.94*31;
   f[11]<-0.79*30;f[12]<-0.74*31} else
 if(latitud==47)
   {f[1]<-0.77*31;f[2]<-0.8*28;f[3]<-1.02*31;f[4]<-1.14*30;f[5]<-1.3*31;
   f[6]<-1.32*30;f[7]<-1.33*31;f[8]<-1.22*31;f[9]<-1.04*30;f[10]<-0.93*31;
   f[11]<-0.78*30;f[12]<-0.73*31} else
 if(latitud==48)
   {f[1]<-0.76*31;f[2]<-0.8*28;f[3]<-1.02*31;f[4]<-1.14*30;f[5]<-1.31*31;
   f[6]<-1.33*30;f[7]<-1.34*31;f[8]<-1.23*31;f[9]<-1.05*30;f[10]<-0.93*31;
   f[11]<-0.77*30;f[12]<-0.72*31} else
 if(latitud==49)
   {f[1]<-0.75*31;f[2]<-0.79*28;f[3]<-1.02*31;f[4]<-1.14*30;f[5]<-1.32*31;
   f[6]<-1.34*30;f[7]<-1.35*31;f[8]<-1.24*31;f[9]<-1.05*30;f[10]<-0.93*31;
   f[11]<-0.76*30;f[12]<-0.71*31} else
 if(latitud>=50)
   {f[1]<-0.74*31;f[2]<-0.78*28;f[3]<-1.02*31;f[4]<-1.15*30;f[5]<-1.33*31;
   f[6]<-1.36*30;f[7]<-1.37*31;f[8]<-1.25*31;f[9]<-1.06*30;f[10]<-0.92*31;
   f[11]<-0.76*30;f[12]<-0.70*31}

### Vamos con los cálculos de la ETP según Thorntwaite ###
## Índice de calor mensual correspodiente a las temperaturas medias mensuales ##
 for(i in 1:12)
  {if(tm[i]<=0) (s[i]<-0) else s[i]<-(tm[i]/5)^1.514}
 for(i in 1:12)
  {Lr<-Lr+s[i]}
 ar<-(0.000000675*(Lr^3))-(0.0000771*(Lr^2))+(0.01792*Lr)+(0.49239)
## Ya tenemos todos los factores que necesitamos, pasamos a calcular «e»
 for(i in 1:12)
#Si la temperatura media del mes es igual o menor que cero, «e» es cero
  {if(tm[i]<=0) e[i]<-0 else
#Si va de más de cero hasta menos de 26.5, se calcula
  if(0<tm[i] && tm[i]<26.5) e[i]<-(1.6*((10*tm[i])/Lr)^ar) else
#A partir de 26,5º el cálculo de e[i] es directo por esta fórmula
if(tm[i]>=26.5)
e[i]<- (-(0.42*((tm[i])^2))+31.49*(tm[i])-404.61)/10}



# Sin más dilaciones, se procede ya al cálculo de la etp mensual
 for(i in 1:12){
 if(tm[i]<=0) (etp[i]==0.0) else (etp[i]<-(f[i]*e[i])/3)}

################## Calculos etr, vr,r, s, d ############
####Primer mes del año hidrológico (el nuevo) ###
if(etp[9]>=pm[9]){
etr[9]<-pm[9];r[9]<-0;vr[9]<-0;sp[9]<-0;d[9]<-(etp[9]-pm[9])} 
else {etr[9]<-etp[9];vr[9]<-(pm[9]-etr[9]);r[9]<-vr[9];d[9]<-0}
#Para el caso de que r>200(100) recalculamos
if(r[9]>res) {sp[9]<-(r[9]<-res);r[9]<-res;vr[9]<-res} else {sp[9]<-0}
#########################################################

#Meses 10 a 12
for(i in 10:12){ 
#Caso a: más lluvia que etp
if(pm[i]>=etp[i]) {etr[i]<-etp[i];vr[i]<-pm[i]-etp[i];r[i]<-r[i-1]+vr[i];d[i]<-0;
if(r[i]>res) {sp[i]<-(r[i]-res);r[i]<-res;vr[i]<-res-r[i-1]} else sp[i]<-0}
else
#Caso b: menos lluvia que etp
if(pm[i]<etp[i]){
#Caso b1: p+reserva es mayor o igual que etp
if(pm[i]+r[i-1]>=etp[i] && pm[i]<etp[i])
{etr[i]<-etp[i];vr[i]<-pm[i]-etr[i];r[i]<-r[i-1]+vr[i];sp[i]<-pm[i]-etr[i];d[i]<-0;
if(r[i]<0) r[i]<-0;
if(r[i]>res) {sp[i]<-(r[i]-res);r[i]<-res;vr[i]<-0} else sp[i]<-0}
#Caso b: p+reserva es menor que etp
else
{etr[i]<-pm[i]+r[i-1];vr[i]<-(-r[i-1]);r[i]<-0;sp[i]<-0;d[i]<-etp[i]-etr[i];
}}}
#}

#Cálculos para el mes 1, con referencia al mes 12
#Caso a: más lluvia que etp
if(pm[1]>=etp[1]) {etr[1]<-etp[1];vr[1]<-pm[1]-etp[1];r[1]<-r[12]+vr[1];d[1]<-0;
if(r[1]>res) {sp[1]<-(r[1]-res);r[1]<-res;vr[1]<-res-r[12]} else sp[1]<-0
}
else {
#Caso b: menos lluvia que etp
#Caso b1: p+reserva es mayor o igual que etp

if(pm[1]+r[12]>=etp[1]){
etr[1]<-etp[1];vr[1]<-pm[1]-etr[1];r[1]<-r[12]+vr[1];sp[1]<0;d[1]<-0;
if(r[1]<0) r[1]<-0;
#if(r[1]>res) {sp[1]<-(r[1]-200);r[1]<-200;vr[1]<-0} else sp[1]<-0
}

#Caso b: p+reserva es menor que etp
if(pm[1]+r[12]<etp[i])
{etr[1]<-pm[1]+r[12];vr[1]<-(-r[12]);r[1]<-0;sp[1]<-0;d[1]<-etp[1]-etr[1];
if(r[1]<0) r[1]<-0}}

#Meses 2 a 8
for(i in 2:8) {
#Caso a: más lluvia que etp
if(pm[i]>=etp[i]) {etr[i]<-etp[i];vr[i]<-pm[i]-etp[i];r[i]<-r[i-1]+vr[i];d[i]<-0;
if(r[i]>res) {sp[i]<-(r[i]-res);r[i]<-res;vr[i]<-res-r[i-1]} else sp[i]<-0}
else 
#Caso b: menos lluvia que etp
#Caso b1: p+reserva es mayor o igual que etp
if(pm[i]+r[i-1]>=etp[i])
{
etr[i]<-etp[i];vr[i]<-pm[i]-etr[i];
r[i]<-r[i-1]+vr[i];sp[i]<-0;d[i]<-0} 
else
#Caso b: p+reserva es menor que etp
#if(pm[i]+r[i-1]<etp[i]
{etr[i]<-pm[i]+r[i-1];vr[i]<- -r[i-1];r[i]<-0;sp[i]<-0;d[i]<-etp[i]-etr[i]
}}
###########################################################################
###########################################################################

pmax <- max(pm) #precipitación máxima
  ymax <- 60  #máxima ordenada por defecto
cat("ymax=",ymax,"\n");
 # if(pmax > 300) ymax <- 50 + 10*floor((pmax+100)/200)
  if((pmax > 250) && (pmax<=300)) ymax <- 60 + 10*floor((pmax+100)/200)
  if((pmax > 300) && pmax<=800) ymax <- 50 + 10*floor((pmax+100)/200)
  if(pmax > 800) ymax <- 60 + 10*floor((pmax+100)/200) 
cat("pmax=",pmax,"ymax=",ymax,"\n");
  ymin <- min(-1.5,min(tm)) #mínima ordenada sin redondear
  #ejes:
  if(ymin < -1.5) {
    ymin=floor(ymin/10)*10 #mínima ordenada redondeada
cat("ymin=",ymin,"\n")
    labT <- paste(ymin)
    labP <- ""
    if(ymin < -10) {
      for(i in (ymin/10+1):-1) {
        labT <- c(labT,i*10)
        labP <- c(labP,"")
cat(i,i*10,"\n",labT,"\n",labP,"\n")
      }
    }
    labT <- c(labT,"0","10","20","30","40","50","")
    labP <- c(labP,"0","20","40","60","80","100","300")
  }
  else {
    labT <- c("0","10","20","30","40","50","")
    labP <- c("0","20","40","60","80","100","300")
  }
  if(ymax > 60) {
    for(i in 6:(ymax/10-1)) {
      labT <- c(labT,"")
      labP <- c(labP,100*(2*i-7))
    }
  }
m<-matrix(c(1,1,1,1,1,1,2,3),4,2,byrow=T)
layout(m)
#layout(matrix(c(1,1,1,1,2,2),6,4))
#layout(matrix(c(1,1,1,1,2,2),6,6,byrow=T))
#Empezamos con la línea de temperaturas
plot(0:13-0.5,c(tm[12],tm[1:12],tm[1]),xlim=c(0,12),ylim=c(ymin,ymax),type="l",col=tcol,lty="dotted",xaxs="i",yaxs="i",xaxp=c(0,12,12),xlab="",ylab="",xaxt="n",yaxt="n",bty="n",main=titulo,bty="n")

x <- 0:13-0.5

p2<-c(pm[12],pm[1:12],pm[1])/2
d2<-c(d[12],d[1:12],d[1])
vr2<-c(vr[12],vr[1:12],vr[1])
sp2<-c(sp[12],sp[1:12],sp[1])
for(i in 1:14) {
if(p2[i]>50) p2[i]=50+(p2[i]-50)/10}

etp2<-c(etp[12],etp[1:12],etp[1])/2
for(i in 1:14) {
if(etp2[i] >50) etp2[i]=50+(etp2[i]-50)/10}

#Rutina de relleno de polígonos en el balance
colorpol<-numeric(14)
for(i in 1:14) {
if(d2[i] > 0) colorpol[i]="gold" else #déficit
if(vr2[i]<0) colorpol[i]="orange" else #gasto de reserva
if(sp2[i]>0) colorpol[i]="lightgreen" else colorpol[i]="deepskyblue2" #superavit & aumento reserva
polygon(c(x[i],x[i],x[i+1],x[i+1],x[i]),c(etp2[i],p2[i],p2[i+1],etp2[i+1],etp2[i]),col=colorpol[i],border=colorpol[i])} #Se pueden añadir: density=60,angle=90
########################################




#Líneas de p y etp
lines(0:13-0.5,c(tm[12],tm[1:12],tm[1]),col="red",lty="dotted",lwd=2)
lines(0:13-0.5,p2,col=pcol,lwd=2)
lines(0:13-0.5,etp2,col=etpcol,lwd=2)

 
#Construcción de los ejes
  lmin <- ymin #mínima ordenada a rotular
  if(lmin==-1.5) lmin=0
  axis(2,((lmin/10):(ymax/10))*10,labels=labT,col.axis=tcol)
  axis(4,((lmin/10):(ymax/10))*10,labels=labP,col.axis=pcol)
  mtext("T \\deC",2,col=tcol,las=1,line=3,adj=0,at=55)
  mtext("P mm",4,col=pcol,las=1,line=3,adj=0.8,at=55)
  if(ymin< -1.5){
	abline(ymin+1.5,0);
	abline(0,0,lty="dotted")}
	else abline(0,0)
  if((max(pm)>100) || (max(etp)>100)) abline(50,0,lty="dotted")

  #rótulos:
  if(hem==1) z<-"N" else z<-"S"
  if(long3==1) zz<-"E" else zz<-"W" 
  mtext(paste(est," [",alt," m], ",lat1,"\\de ",lat2,"' ",z,", ",long1,"\\de ",long2,"' ",zz,sep=""),line=0.5,adj=0.5)
mtext(paste("T: ",Tm,"\\deC; P: ",Pm," mm",sep=""),line=-1,adj=0.05)

  #marcar límites de los meses:
  for(i in 0:13)
	if(lmin> -1.5)
	segments(i,0,i,-1.5) else
	segments(i,lmin+1.5,i,lmin)

  #rótulos meses:
  mtext(mlab,1,las=1,line=0.5,adj=0.5,at=x[2:13])



if(lang=="en") deficit<-"Deficit" else deficit<-"D\\'eficit"
if(lang=="en") gasto<-"Decrease R" else gasto<-"Baja R"
if(lang=="en") incremento<-"Increase R" else incremento<-"Sube R"
if(lang=="en") saturacion<-"Saturation" else saturacion<-"Saturado"


#Leyenda1
margen=c(2,7,1,2)
par(mar=margen,pty="s",las=1,new=FALSE)
plot.new()
rect(0,0.05,0.3,0.25,col="gold")
text(0.40,0.15,deficit,adj=0,font=2)
rect(0,0.3,0.3,0.5,col="orange")
text(0.40,0.4,gasto,adj=0,font=2)
rect(0,0.55,0.3,0.75,col="deepskyblue2")
text(0.40,0.65,incremento,adj=0,font=2)
rect(0,0.8,0.3,1,col="lightgreen")
text(0.40,0.9,saturacion,adj=0,font=2)

#Leyenda2
margen=c(0.1,0,0.1,0)
par(mar=margen,pty="s",las=1,new=FALSE)
plot.new()
segments(0.1,0.3,0.4,0.3,col="red",lty="dotted",lwd=2)
text(0.5,0.3,"T",font=2,cex=1,adj=0)
segments(0.1,0.5,0.4,0.5,col="blue",lwd=2)
text(0.5,0.5,"P",font=2,cex=1,adj=0)
segments(0.1,0.7,0.4,0.7,col="black",lwd=2)
text(0.5,0.7,"ETP",font=2,cex=1,adj=0)

}